@@ -1,8 +1,15 @@
 #'@title Load an OBJ file
 #'
-#'@keywords internal
+#'@description Loads an OBJ file and return a `ray_mesh` list structure. No processing is done on
+#'the object other than loading it (unlike `obj_model()`).
 #'
-#'@return Rasterized image.
+#'@param filename Filename of the OBJ file.
+#'@param materialspath Directory where the MTL file is located. Defaults to the directory of `filename`.
+#'@export
+#'
+#'@return `ray_mesh` list object 
+#'#Load an arrow OBJ
+#'sphere = read_obj(system.file("extdata", "arrow.txt", package="rayvertex"))
 read_obj = function(filename, materialspath = NULL) {
   filename = path.expand(filename)
   sepval = ""
@@ -28,6 +35,29 @@ read_obj = function(filename, materialspath = NULL) {
       obj_loaded$shapes[[i]]$has_vertex_normals[apply(obj_loaded$shapes[[i]]$norm_indices,1,(function(x) any(x == -1)))] = FALSE
     }
   }
+  single_indices            = list()
+  single_tex_indices        = list()
+  single_norm_indices       = list()
+  single_material_ids       = list()
+  single_has_vertex_tex     = list()
+  single_has_vertex_normals = list()
+  
+  for(i in seq_len(length(obj_loaded$shapes))) {
+    single_indices[[i]]            = obj_loaded$shapes[[i]]$indices
+    single_tex_indices[[i]]        = obj_loaded$shapes[[i]]$tex_indices
+    single_norm_indices[[i]]       = obj_loaded$shapes[[i]]$norm_indices
+    single_material_ids[[i]]       = obj_loaded$shapes[[i]]$material_ids
+    single_has_vertex_tex[[i]]     = obj_loaded$shapes[[i]]$has_vertex_tex
+    single_has_vertex_normals[[i]] = obj_loaded$shapes[[i]]$has_vertex_normals
+  }
+  
+  obj_loaded$shapes = list(list(indices = do.call(rbind,single_indices),
+                           tex_indices = do.call(rbind,single_tex_indices),
+                           norm_indices = do.call(rbind,single_norm_indices),
+                           material_ids = do.call(rbind,single_material_ids),
+                           has_vertex_tex = do.call(c,single_has_vertex_tex),
+                           has_vertex_normals = do.call(c,single_has_vertex_normals)))
+
   for(i in seq_len(length(obj_loaded$materials))) {
     if(!file.exists(obj_loaded$materials[[i]]$diffuse_texname) && nchar(obj_loaded$materials[[i]]$diffuse_texname) > 0 &&
        file.exists(sprintf("%s%s%s", dir,sepval,obj_loaded$materials[[i]]$diffuse_texname))) {
@@ -55,6 +85,7 @@ read_obj = function(filename, materialspath = NULL) {
     hashes[i] = digest::digest(obj_loaded$materials[[i]])
   }
   obj_loaded$material_hashes = hashes
+  class(obj_loaded) = c("ray_mesh", "list")
   obj_loaded
 }
 
@@ -88,5 +119,6 @@ read_ply = function(filename) {
     hashes[i] = digest::digest(ply_loaded$materials[[i]])
   }
   ply_loaded$material_hashes = hashes
+  class(ply_loaded) = c("ray_mesh","list")
   ply_loaded
 }
\ No newline at end of file
